@@ -1,54 +1,10 @@
 '''Functions related to exporting the dataset to the Deep Graph Library'''
 
-import multiprocessing as mp
-from typing import Dict, Tuple, Callable
-from tqdm.auto import tqdm
+from typing import Dict, Tuple, Union
+from pathlib import Path
 import pandas as pd
 import numpy as np
 import json
-from functools import partial
-
-
-def parallel_map(series: pd.Series,
-                 func: Callable,
-                 n_jobs: int = -1,
-                 progress_bar: bool = True) -> list:
-    '''Apply a function to a Pandas Series in parallel.
-
-    Args:
-        series (pd.Series):
-            The Pandas Series to apply the function to.
-        func (callable):
-            The function to apply to each element of the Pandas Series.
-        n_jobs (int, optional):
-            The number of parallel jobs to run. Defaults to -1, which uses all
-            available cores.
-        progress_bar (bool, optional):
-            Whether to show a progress bar. Defaults to True.
-
-    Returns:
-        pd.Series:
-            The resulting Pandas Series.
-    '''
-    # Set `n_jobs` to be the number of cores if it is set to -1
-    if n_jobs == -1:
-        n_jobs = mp.cpu_count()
-
-    # Create a pool of workers and apply the function to each element of the
-    # Pandas Series
-    results = list()
-    with mp.Pool(n_jobs) as pool:
-        with tqdm(series, leave=False, disable=(not progress_bar)) as pbar:
-            for result in pool.imap(func, pbar, chunksize=100):
-                results.append(result)
-
-    # Concatenate the results to a Pandas Series and return it
-    return results
-
-
-def change_index(old_idx: int, df: pd.DataFrame) -> int:
-    new_idx = df.where(df == old_idx).dropna().index[0]
-    return new_idx
 
 
 def build_dgl_dataset(nodes: Dict[str, pd.DataFrame],
@@ -90,43 +46,67 @@ def build_dgl_dataset(nodes: Dict[str, pd.DataFrame],
 
     # Set up the graph as a DGL graph
     graph_data = dict()
-    desc = 'Setting up MuMiN as a DGL graph'
-    with tqdm(list(relations.items()), desc=desc) as pbar:
-        for canonical_etype, rel_arr in pbar:
+    for canonical_etype, rel_arr in relations.items():
 
-            # Update the progress bar description
-            pbar.set_description(desc + f' - {canonical_etype}')
+        # Drop the NaN nodes, corresponding to the deleted tweets. We also
+        # reset the indices to start from 0, as DGL requires there to be no
+        # gaps in the indexing
+        src, rel, tgt = canonical_etype
+        allowed_src = (nodes[src].dropna()
+                                 .reset_index()
+                                 .rename(columns=dict(index='old_idx'))
+                                 .old_idx)
+        allowed_src = {old: new for new, old in allowed_src.iteritems()}
+        allowed_tgt = (nodes[tgt].dropna()
+                                 .reset_index()
+                                 .rename(columns=dict(index='old_idx'))
+                                 .old_idx)
+        allowed_tgt = {old: new for new, old in allowed_tgt.iteritems()}
 
-            src, rel, tgt = canonical_etype
-            allowed_src = (nodes[src].dropna()
-                                     .reset_index()
-                                     .rename(columns=dict(index='idx'))
-                                     .idx)
-            allowed_tgt = (nodes[tgt].dropna()
-                                     .reset_index()
-                                     .rename(columns=dict(index='idx'))
-                                     .idx)
-            rel_arr = (relations[canonical_etype][['src', 'tgt']]
-                       .query('src in @allowed_src.tolist() and '
-                              'tgt in @allowed_tgt.tolist()')
-                       .drop_duplicates())
-            rel_arr.src = (parallel_map(rel_arr.src,
-                                        partial(change_index, df=allowed_src)))
-            rel_arr.tgt = (parallel_map(rel_arr.tgt,
-                                        partial(change_index, df=allowed_tgt)))
-            rel_arr = rel_arr.to_numpy()
-            if rel_arr.size:
-                src_tensor = torch.from_numpy(rel_arr[:, 0]).int()
-                tgt_tensor = torch.from_numpy(rel_arr[:, 1]).int()
-                graph_data[canonical_etype] = (src_tensor, tgt_tensor)
+        # Get a dataframe containing the edges between allowed source and
+        # target nodes (i.e., non-deleted)
+        rel_arr = (relations[canonical_etype][['src', 'tgt']]
+                   .query('src in @allowed_src.values() and '
+                          'tgt in @allowed_tgt.values()')
+                   .drop_duplicates())
 
-                # Adding inverse relations as well, to ensure that graph is
-                # bidirected
-                graph_data[(tgt, f'{rel}_inv', src)] = (tgt_tensor, src_tensor)
+        # Convert the node indices in the edge dataframe to the new indices
+        # without gaps
+        rel_arr.src = [allowed_src[old_idx]
+                       for old_idx in rel_arr.src.tolist()]
+        rel_arr.tgt = [allowed_tgt[old_idx]
+                       for old_idx in rel_arr.tgt.tolist()]
 
+        # Convert the edge dataframe to a NumPy array
+        rel_arr = rel_arr.to_numpy()
+
+        # If there are edges left in the edge array, then convert these to
+        # PyTorch tensors and add them to the graph data
+        if rel_arr.size:
+            src_tensor = torch.from_numpy(rel_arr[:, 0]).int()
+            tgt_tensor = torch.from_numpy(rel_arr[:, 1]).int()
+            graph_data[canonical_etype] = (src_tensor, tgt_tensor)
+
+            # Adding inverse relations as well, to ensure that graph is
+            # bidirected
+            graph_data[(tgt, f'{rel}_inv', src)] = (tgt_tensor, src_tensor)
+
+    # Initialise a DGL heterogeneous graph from the graph data
     dgl_graph = dgl.heterograph(graph_data)
 
-    def emb_to_tensor(df: pd.DataFrame, col_name: str):
+    def emb_to_tensor(df: pd.DataFrame, col_name: str) -> torch.Tensor:
+        '''Convenience function converting embeddings to tensors.
+
+        Args:
+            df (pd.DataFrame):
+                The dataframe containing the embeddings.
+            col_name (str):
+                The name of the column containing the embeddings.
+
+        Returns:
+            torch.Tensor:
+                The embeddings as a PyTorch tensor.
+        '''
         if type(df[col_name].iloc[0]) == str:
             df[col_name] = df[col_name].map(lambda x: json.loads(x))
         np_array = np.stack(df[col_name].tolist())
@@ -258,14 +238,14 @@ def build_dgl_dataset(nodes: Dict[str, pd.DataFrame],
     return dgl_graph
 
 
-def save_dgl_graph(dgl_graph, path: str = 'mumin-small-dgl.bin'):
+def save_dgl_graph(dgl_graph, path: Union[str, Path] = 'mumin.dgl'):
     '''Save a MuMiN DGL graph.
 
     Args:
         dgl_graph (DGL heterogeneous graph):
             The graph to store.
         path (str, optional):
-            Where to store the graph. Defaults to 'mumin-small-dgl.bin'.
+            Where to store the graph. Defaults to 'mumin.dgl'.
     '''
     # Import the needed libraries, and raise an error if they have not yet been
     # installed
@@ -285,15 +265,15 @@ def save_dgl_graph(dgl_graph, path: str = 'mumin-small-dgl.bin'):
             dgl_graph.nodes[node_type].data[mask_name] = t.type(torch.uint8)
 
     # Save the graph
-    save_graphs(path, [dgl_graph])
+    save_graphs(str(path), [dgl_graph])
 
 
-def load_dgl_graph(path: str = 'mumin-small-dgl.bin'):
+def load_dgl_graph(path: Union[str, Path] = 'mumin.dgl'):
     '''Load a MuMiN DGL graph.
 
     Args:
-        path (str, optional):
-            Where to load the graph from. Defaults to 'mumin-small-dgl.bin'.
+        path (str or Path, optional):
+            Where to load the graph from. Defaults to 'mumin.dgl'.
 
     Returns:
         DGLHeteroGraph:
@@ -310,7 +290,7 @@ def load_dgl_graph(path: str = 'mumin-small-dgl.bin'):
                                   'mumin[dgl]`')
 
     # Load the graph
-    dgl_graph = load_graphs(path)[0]
+    dgl_graph = load_graphs(str(path))[0][0]
 
     # Convert masks back to booleans
     for mask_name in ['train_mask', 'val_mask', 'test_mask']:
